from __future__ import absolute_import, division, print_function, unicode_literals

import pandas as pd
import math as m
import tensorflow as tf
from tensorflow import feature_column
from tensorflow.keras import layers
from sklearn.model_selection import train_test_split


# A utility method to create a tf.data dataset from a Pandas Dataframe


def df_to_dataset(dataframe, shuffle=True, batch_size=32):
    dataframe = dataframe.copy()
    labels = dataframe.pop('Pose')
    ds = tf.data.Dataset.from_tensor_slices((dict(dataframe), labels))
    if shuffle:
        ds = ds.shuffle(buffer_size=len(dataframe))
    ds = ds.batch(batch_size)
    return ds


tf.keras.backend.set_floatx('float32')

dataframe = pd.read_excel("data.xlsx")

dataframe['Pose'] = pd.factorize(dataframe['Pose'])[0]

dataframe.drop('Frame', axis=1)
dataframe.drop('TranX', axis=1)
dataframe.drop('TranY', axis=1)
dataframe.drop('TranZ', axis=1)
dataframe.drop('ScaleX', axis=1)
dataframe.drop('ScaleY', axis=1)
dataframe.drop('ScaleZ', axis=1)
indexNames = dataframe[(dataframe['RotW'] == 1) & (dataframe['RotX'] == 0) & (dataframe['RotY'] == 0) & (dataframe['RotZ'] == 0) &
                        (dataframe['Pose'] != 0)].index
dataframe.drop(indexNames, inplace=True)

dataframe.to_excel("eng.xlsx")
print(dataframe.head())

train, test = train_test_split(dataframe, test_size=0.2)
train, val = train_test_split(train, test_size=0.2)
print(len(train), 'train examples')
print(len(val), 'validation examples')
print(len(test), 'test examples')

batch_size = 1
train_ds = df_to_dataset(train, shuffle=True, batch_size=batch_size)
val_ds = df_to_dataset(val, shuffle=True, batch_size=batch_size)
test_ds = df_to_dataset(test, shuffle=True, batch_size=batch_size)

feature_columns = []

names = feature_column.categorical_column_with_vocabulary_list(
    'Name', [
        "root",
        "pelvis",
        "thigh_R",
        "calf_R",
        "foot_R",
        "toes_R",
        "calf_twist_R",
        "thigh_twist_R",
        "thigh_L",
        "calf_L",
        "foot_L",
        "toes_L",
        "calf_twist_L",
        "thigh_twist_L",
        "spine01",
        "spine02",
        "spine03",
        "clavicle_L",
        "upperarm_L",
        "lowerarm_L",
        "hand_L",
        "thumb01_L",
        "thumb02_L",
        "thumb03_L",
        "index00_L",
        "index01_L",
        "index02_L",
        "index03_L",
        "middle00_L",
        "middle01_L",
        "middle02_L",
        "middle03_L",
        "ring00_L",
        "ring01_L",
        "ring02_L",
        "ring03_L",
        "pinky00_L",
        "pinky01_L",
        "pinky02_L",
        "pinky03_L",
        "lowerarm_twist_L",
        "upperarm_twist_L",
        "clavicle_R",
        "upperarm_R",
        "lowerarm_R",
        "hand_R",
        "thumb01_R",
        "thumb02_R",
        "thumb03_R",
        "index00_R",
        "index01_R",
        "index02_R",
        "index03_R",
        "middle00_R",
        "middle01_R",
        "middle02_R",
        "middle03_R",
        "ring00_R",
        "ring01_R",
        "ring02_R",
        "ring03_R",
        "pinky00_R",
        "pinky01_R",
        "pinky02_R",
        "pinky03_R",
        "lowerarm_twist_R",
        "upperarm_twist_R",
        "neck",
        "head",
        "breast_L",
        "breast_R",
    ])

names_one_hot = feature_column.indicator_column(names)

rotW = feature_column.numeric_column("RotW")
feature_columns.append(rotW)
rotX = feature_column.numeric_column("RotX")
feature_columns.append(rotX)
rotY = feature_column.numeric_column("RotY")
feature_columns.append(rotY)
rotZ = feature_column.numeric_column("RotZ")
feature_columns.append(rotZ)

feature_layer = tf.keras.layers.DenseFeatures(feature_columns)

model = tf.keras.Sequential([
    feature_layer,
    layers.Dense(512, activation='relu'),
    layers.Dense(512, activation='relu'),
    layers.Dense(4)
])

model.compile(optimizer=tf.keras.optimizers.Adam(learning_rate=0.00001, epsilon=1e-07, name='Adam'),
              loss='sparse_categorical_crossentropy',
              metrics=['accuracy'])

model.fit(train_ds,
          validation_data=val_ds,
          epochs=3)

loss, accuracy = model.evaluate(test_ds)
print("Accuracy", accuracy)
